/* THIS SAMPLE CODE IS PROVIDED AS IS AND IS SUBJECT TO ALTERATIONS. FUJITSU */
/* MICROELECTRONICS ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR */
/* ELIGIBILITY FOR ANY PURPOSES.                                             */
/*                 (C) Fujitsu Microelectronics Europe GmbH                  */
/*---------------------------------------------------------------------------*/

#include "mb90350.h"

/* -----------------------   D E F I N E s -------------------------------------*/

#define BACK 0x08	/* Backspace-Eingabe (HEX-Wert)		*/
#define RET  0x0d	/* Return-Eingabe (HEX-Wert)		*/
#define ESC  0x1b   /* Escape-Eingabe (HEX-Wert)		*/
#define HEX	 0
#define DEC	 1

#define TIMEOUT 50000    /* <<< timeout value, if CAN-data can not be send   */

unsigned char ch;
unsigned int  i;
unsigned long timeout;                   /* timeout-counter                */

unsigned char CAN_Receive_Data[9], DataPos;
unsigned int  CAN_Receive_ID;
unsigned int  buffer, buffercnt ;

/*===========================================================================*/
/*========== UART 3 =========================================================*/
/*===========================================================================*/
  
void InitUart3(void)
{
  // Initialize UART3 asynchronous mode
  BGR3 = 2499; // 9600 Baud @ 24MHz
  SCR3 = 0x17; // 8N1
  SMR3 = 0x0d; // enable SOT2, Reset, normal mode
  SSR3 = 0x00; // LSB first
}

void Putch3(char ch)         // sends one char via UART3
{
  while (SSR3_TDRE == 0);    // wait for transmit buffer empty
  TDR3 = ch;                 // put ch into buffer
}

unsigned char Getch3(void)     /* Waits for and returns incomming char */
{
  unsigned char rxchar;
  
  for(;;)
  {

    while(SSR3_RDRF == 0)      /* Wait for data received  	           */
    {
      __wait_nop(); 
    }

    rxchar = RDR3;             /* Save receive register                */

    if ((SSR3 & 0xE0) != 0)    /* Check for errors PE, ORE, FRE        */
    {
      SCR3_CRE  = 0;           /* Clear error flags                    */
    }
    else
    {
      return (rxchar);         /* Return received character            */
    }

  }
  
}

void Puts3(const char *string)     // send a string via UART3
{ 
  int i,len;
 
  len = strlen(string);
	
  for (i=0; i<strlen(string); i++)     // go through string
  {
    if (string[i] == 10)               // LF =>
      Putch3(13);                      //       insert CR
    Putch3(string[i]);                 // send it out
  }
}


/***************************************************************************
* unsigned int hex2int (void)
*
* DESCRIPTION: Eingabe eines HEX-Wertes
***************************************************************************/
unsigned int hex2int () 
{
	unsigned int help;
	unsigned char help_char;
	help = 0;
	do {} while(!(int)SSR3_RDRF); 	 /* Wait until a character was received. */
	help_char = RDR3;
	while (help_char > 13)
	{
		Putch3(help_char);
		if (help_char >= 97) 
		{ 
			help = help * 16 + (help_char - 87); 
		}
 		else 
		{ 
			if (help_char >= 65) 
			{ 	
				help = help * 16 + (help_char - 55);
			}
			else
			{
				help = help * 16 + (help_char - 48);
			}
		}
	  do {} while(!(int)SSR3_RDRF);   /* Wait until a character was received. */
	  help_char = RDR3;
	}
	return(help);             /* If no error: return received character. */
}

/***************************************************************************
* long hex2long (void)
*
* DESCRIPTION: Eingabe eines HEX-Wertes
***************************************************************************/
unsigned long hex2long () 
{
	long help;
	unsigned char help_char;
	help = 0;
	do {} while(!(int)SSR3_RDRF); 			/* Wait until a character was received. */
	help_char = RDR3;
	while (help_char != 13)
	{
		Putch3(help_char);
		if (help_char >= 97) 
		{ 
			help = help * 16 + (help_char - 87); 
		}
 		else 
		{ 
			if (help_char >= 65) 
			{ 	
				help = help * 16 + (help_char - 55);
			}
			else
			{
				help = help * 16 + (help_char - 48);
			}
		}
	do {} while(!(int)SSR3_RDRF); 			/* Wait until a character was received. */
	help_char = RDR3;
	}
	return(help);             /* If no error: return received character. */
}


void binasci4(unsigned int a, unsigned char base, char fill)				
{							
  unsigned char val;
  a &= 0x0F;	

  switch(base)
	{	
		case HEX:	if (a <= 9) Putch3(a + '0'); 					   
  				else        Putch3(a + 'A' - 10);
				break;

		case DEC:	val = a / 10;
				if( val ) Putch3( a / 10 + '0');
				    else  Putch3( fill );
				Putch3( a % 10 + '0');
				break;

		default:	Puts3(" <format not supported> ");
	}
}


void binasci8(unsigned int a, unsigned char base, char fill)
{
  unsigned int rem,val,d;
  unsigned char filling=1;
								
	switch(base)
	{	
		case HEX:	binasci4(a >> 4, HEX, fill);				     	
  				binasci4(a, HEX, fill);						
				break;

		case DEC:	rem = a;
				for( d=100; d >= 10; d = d / 10)
				{ 
				  val = rem / d; rem = rem % d;
				  if( val ) filling=0;
				  if( filling ) Putch3( fill );
					else    Putch3( val + '0' );
				}
				Putch3( rem + '0');
				break;

		default:	Puts3(" <format not supported> ");
	}

}


void binasci16(unsigned int a, unsigned char base, char fill)
{											              
  unsigned int rem,val,d;
  unsigned char filling=1;

  switch(base)
	{	
		case HEX:	binasci8(a >> 8, HEX, fill);				     	
  				binasci8(a, HEX, fill);						
				break;

		case DEC:	rem = a;
				for( d=10000; d >= 10; d = d / 10)
				{ 
				  val = rem / d; rem = rem % d;
				  if( val ) filling=0;
				  if( filling ) Putch3( fill );
					else    Putch3( val + '0' );
				}
				Putch3( rem + '0');
				break;

		default:	Puts3(" <format not supported> ");
	}

}
	

void binasci32( unsigned long a, unsigned char base, char fill)
{
  unsigned long rem,val,d;
  unsigned char filling=1;


  switch(base)
	{	
		case HEX:	binasci16(a >> 16, HEX, fill);				     	
  				binasci16(a, HEX, fill);						
				break;

		case DEC:	rem = a;
				for( d=1000000000; d >= 10; d = d / 10)
				{ 
				  val = rem / d; rem = rem % d;
				  if( val ) filling=0;
				  if( filling ) Putch3( fill );
					else    Putch3( val + '0' );
				}
				Putch3( rem + '0');
				break;

		default:	Puts3(" <format not supported> ");
	}

}

/*===========================================================================*/
/*====== CAN ================================================================*/
/*===========================================================================*/
/*       Search for "<<<" There changes by the user may be required          */

/*===== CAN ======= */

#define BTR_16M_20k8_24_66_2 0x7EDF /* BTR config 20.8 kBaud @ 16MHz */
#define BTR_16M_33K0_22_68_3 0x6DD5 /* BTR config 33.0 kBaud @ 16MHz */
#define BTR_16M_33K3_22_68_3 0x4DD5 /* BTR config 33.3 kBaud @ 16MHz */
#define BTR_16M_83K3_24_66_2 0x7EC7 /* BTR config 83.3 kBaud @ 16MHz */
#define BTR_16M_100k_20_70_3 0x5CC7 /* BTR config  100 kBaud @ 16MHz */
#define BTR_16M_125k_16_68_2 0x4987 /* BTR config  125 kBaud @ 16MHz */
#define BTR_16M_500k_16_68_2 0x4981 /* BTR config  500 kBaud @ 16MHz */
#define BTR_16M_1M00_16_68_2 0x4980 /* BTR config  1.0 MBaud @ 16MHz */

#define	BTR_20M_100k_20_70_3 0x5CC9 /* BTR config  100 kBaud @ 20MHz */

#define	BTR_24M_100k_20_70_3 0x5CCB /* BTR config  100 kBaud @ 24MHz */

// The following macros convert from identifier value to the appropriate
// I/O register value and vise versa. These macros should be used for 
// constant identifier values.
// The actual order of ID bits in the register can be found in the manual.
// ID (11bit-IDs uses Bit 28..18
//    (29bit-IDs uses Bit 28...0

#define MSG2STD(msg) ( (unsigned short )(msg) >>  3 & 0x000000FF  \
                     | (unsigned short )(msg) << 13 & 0x0000FF00  )

#define MSG2EXT(msg) ( (unsigned long  )(msg) >> 21 & 0x000000FF  \
                     | (unsigned long  )(msg) >>  5 & 0x0000FF00  \
                     | (unsigned long  )(msg) << 11 & 0x00FF0000  \
                     | (unsigned long  )(msg) << 27 & 0xFF000000  )

#define STD2MSG(reg) ( ((unsigned short)(reg) & 0x000000FF) <<  3  \
                     | ((unsigned short)(reg) & 0x0000FF00) >> 13  )

#define EXT2MSG(reg) ( ((unsigned long )(reg) & 0x000000FF) << 21  \
                     | ((unsigned long )(reg) & 0x0000FF00) <<  5  \
                     | ((unsigned long )(reg) & 0x00FF0000) >> 11  \
                     | ((unsigned long )(reg) & 0xFF000000) >> 27  )

void CAN_init(void)
{
  CSR1        = 0x81;                 /* bit 7: enable transmit output pin   */
                                      /* bit 0: stop bus operation           */
  while (!CSR1_HALT);
  
  TCANR1      = 0xffff;               /* cancel all Transmissions            */
  BVALR1      = 0;                    /* all message buffer are invalid      */
  BTR1        = BTR_24M_100k_20_70_3; /* <<< set bit timing                  */
  CSR1        = 0x0080;               /* activate can bus operation          */

  Puts3("\nCAN Connecting...");
  while (CSR1_HALT);
  Puts3("ok!\n\n");
    
  AMR01       = 0;                    /* <<< acceptance mask definition 0    */
  AMR11       = 0;                    /* <<< acceptance mask definition 1    */
}


/*===== CAN Buffer ======= */

/* it is recommended not to use Buffer0 in general, 
   in order to prevent from Hit&Away trap           */

#define buffer1 1        /* Messagebuffer 1 */

void CAN_buffer1(void)
{
  BVALR1_BVAL1 = 0;      /* message buffer are invalid                       */

  IDER1_IDE1   = 0;      /* <<< 0:11bit 1:29bit indentifier                  */

  IDR1(1)      = 0x2000; /* <<< ID e.g. 0x2000 = ID #1                       */
                         /*        (11bit-IDs uses Bit 28..18                */
                         /*        (29bit-IDs uses Bit 28...0                */
                         /* see Utility Bitmixer
                         /* or use defines: e.g. IDR(1) = MSG2STD(1);        */

  AMSR1_AMS1   = 0;      /* <<< Acceptance Mask:                             */
                         /* 0 : full bit compare (full can)                  */
                         /* 1 : full bit mask (basic can)                    */
                         /* 2 : use acceptance mask 0                        */
                         /* 3 : use acceptance mask 1                        */

  TIER1_TIE1   = 0;      /* <<< 0: diasble / 1: enable Trx-Interrupt         */

  RIER1_RIE1   = 1;      /* <<< 0: diasble / 1: enable Rx-Interrupt          */

  RFWTR1_RFWT1 = 0;      /* <<< 0: no wait for remote requeset               */

  TRTRR1_TRTR1 = 0;      /* <<< 0: Data / 1: Remote frame Trx                */

  BVALR1_BVAL1 = 1;      /* message buffer are valid                         */
}


#define buffer2 2        /* Messagebuffer 2 */

void CAN_buffer2(void)
{
  BVALR1_BVAL2 = 0;      /* message buffer are invalid                       */

  IDER1_IDE2   = 0;      /* <<< 0:11bit 1:29bit indentifier                  */

  IDR1(2)      = 0x0080; /* <<< ID e.g. 0x0080 = MSB set                     */
                         /*        (11bit-IDs uses Bit 28..18                */
                         /*        (29bit-IDs uses Bit 28...0                */
                         /* see Utility Bitmixer
                         /* or use defines: e.g. IDR(1) = MSG2STD(1);        */

  AMSR1_AMS2   = 2;      /* <<< Acceptance Mask:                             */
                         /* 0 : full bit compare (full can)                  */
                         /* 1 : full bit mask (basic can)                    */
                         /* 2 : use acceptance mask 0                        */
                         /* 3 : use acceptance mask 1                        */

  TIER1_TIE2   = 0;      /* <<< 0: diasble / 1: enable Trx-Interrupt         */

  RIER1_RIE2   = 1;      /* <<< 0: diasble / 1: enable Rx-Interrupt          */

  RFWTR1_RFWT2 = 0;      /* <<< 0: no wait for remote requeset               */

  TRTRR1_TRTR2 = 0;      /* <<< 0: Data / 1: Remote frame Trx                */

  BVALR1_BVAL2 = 1;      /* message buffer are valid                         */
}


#define buffer5 5        /* Messagebuffer 5 */

void CAN_buffer5(void)
{
  BVALR1_BVAL5 = 0;      /* message buffer are invalid                       */

  IDER1_IDE5   = 0;      /* <<< 0:11bit 1:29bit indentifier                  */

  IDR1(5)      = 0x0080; /* <<< ID e.g. 0x0080 = MSB set                     */
                         /*        (11bit-IDs uses Bit 28..18                */
                         /*        (29bit-IDs uses Bit 28...0                */
                         /* see Utility Bitmixer
                         /* or use defines: e.g. IDR(1) = MSG2STD(1);        */

  AMSR1_AMS5   = 1;      /* <<< Acceptance Mask:                             */
                         /* 0 : full bit compare (full can)                  */
                         /* 1 : full bit mask (basic can)                    */
                         /* 2 : use acceptance mask 0                        */
                         /* 3 : use acceptance mask 1                        */

  TIER1_TIE5   = 0;      /* <<< 0: diasble / 1: enable Trx-Interrupt         */

  RIER1_RIE5   = 1;      /* <<< 0: diasble / 1: enable Rx-Interrupt          */

  RFWTR1_RFWT5 = 0;      /* <<< 0: no wait for remote requeset               */

  TRTRR1_TRTR5 = 0;      /* <<< 0: Data / 1: Remote frame Trx                */

  BVALR1_BVAL5 = 1;      /* message buffer are valid                         */
}


#define buffer6 6        /* Messagebuffer 6 will be use as send-buffer       */

void CAN_buffer6(void)
{
  /* CAN: messagebuffer 6 will be used as send-buffer                        */
  
  BVALR1_BVAL6 = 0;      /* message buffer are invalid                       */

  IDER1_IDE6   = 0;      /* <<< 0:11bit 1:29bit indentifier                  */

  IDR1(6)      = 0x2000; /* <<< ID e.g. 0x2000 = ID #1                       */
                         /*        (11bit-IDs uses Bit 28..18                */
                         /*        (29bit-IDs uses Bit 28...0                */
                         /* see Utility Bitmixer
                         /* or use defines: e.g. IDR(1) = MSG2STD(1);        */

  AMSR1_AMS6   = 0;      /* <<< Acceptance Mask:                             */
                         /* 0 : full bit compare (full can)                  */
                         /* 1 : full bit mask (basic can)                    */
                         /* 2 : use acceptance mask 0                        */
                         /* 2 : use acceptance mask 1                        */

  TIER1_TIE6   = 0;       /* <<< 0: diasble / 1: enable Trx-Interrupt         */

  RIER1_RIE6   = 0;      /* <<< 0: diasble / 1: enable Rx-Interrupt          */

  RFWTR1_RFWT6 = 0;      /* <<< 0: no wait for remote requeset               */

  TRTRR1_TRTR6 = 0;      /* <<< 0: Data / 1: Remote frame Trx                */

  BVALR1_BVAL6 = 1;      /* message buffer are valid                         */
}


/*===== Readout Buffer x ======= */

unsigned char GetCanData (char  x, unsigned char *BufPtr)
{
  char Len, pos;
    
  *BufPtr++ = Len = DLCR1(x);
  for (pos=0; pos<=Len;pos++)
    *BufPtr++ = DTR1_BYTE(x, pos);
  return (Len); 
}


/*===========================================================================*/
/*========== MAIN ===========================================================*/
/*===========================================================================*/

void main (void)
{
  InitIrqLevels();
  __set_il(7);         /* allow all levels */
  __EI();              /* globaly enable interrupts */

  InitUart3();           /* initialize UART */
  Puts3("\n\nWelcome to the DICE-Kit Evaluation-Board CAN-Demo\n");
  Puts3("H        => help\n\n");
    
  /* initialize CAN */ 
  CAN_init(); 
  CAN_buffer1();
  CAN_buffer2();
  CAN_buffer5();
  CAN_buffer6(); 

  while(1)
  {
    if (SSR3_RDRF)    /* if UART0 received data ...     */
    { 
      ch = RDR3;
      switch (ch)     /* handle on received byte ...    */
      {        
        case 13:   /* "Return" => set new ID-address */
          Puts3("\nEnter ID-address (HEX): ");
          i = hex2int();
          BVALR1_BVAL6 = 0;       /* message buffer is invalid  */
          IDR1(6) = MSG2STD(i); 
          BVALR1_BVAL6 = 1;       /* message buffer is valid  */
          Puts3("   set ! (0x");
          binasci16(STD2MSG(IDR1(6)), HEX, '0');
          Puts3(")\n\n");
          break;
                
        case 'D': /* display CAN message buffers */

          Puts3("\nCAN message buffers\n\r====================\n");
          buffercnt = 0x0001;  
          for (buffer=0; buffer<=7; buffer++)
          {
            Puts3("CAN buffer #"); 
            binasci4(buffer, DEC, ' ');
            Puts3(" :");
            
            if (RCR1 & buffercnt)       /* check if buffer has received message */
            {
              do
              {
                ROVRR1 = ~buffercnt;    /* clear Overrun-flag */

                GetCanData (buffer, CAN_Receive_Data); /* Readout Message Buffer */
                CAN_Receive_ID = IDR1(buffer);
              } 
              while (ROVRR1 & buffercnt);   /* check for overrun: 
                                               Theoretically it may happen that another 
                                               CAN-message is received while data was stored */

              RCR1 = ~buffercnt;            /* clear Reception-flag */
 
              Puts3(" ID: 0x");
              CAN_Receive_ID = STD2MSG(CAN_Receive_ID); /* reordering of ID */
     
              binasci16(CAN_Receive_ID, HEX, '0');
 
              Puts3("   Data: "); 
                   
              for (DataPos=1; DataPos<=CAN_Receive_Data[0]; DataPos++)
              {
                binasci8(CAN_Receive_Data[DataPos], HEX, '0');	// send to uart
                Putch3(32);
              }
            } // if (RCR1 & buffercnt)
            else
            {
              Puts3("*** empty ***");
            }

            Puts3("\n");
            buffercnt = buffercnt << 1;
            
          } // for (buffer=0; buffer<=15; buffer++)
          Puts3("\n\r");
          break;

        case 'H': /* Help-information */
          Puts3("\n\r\n\rHELP:\n\r=====\n\r");
          Puts3("D        => display CAN message buffers\n\r");
          Puts3("H        => help\n\r");
          Puts3("I        => enable/disable CAN-Receive-Interrupts\n\r");
          Puts3("'Return' => Set ID-address for send-message\n\r");
          Puts3("'a-z'    => sends data 'a..z' via CAN buffer #6 to ID\n\r");
          Puts3("\n\r\n\r");
          Puts3("CAN-Buffer 1 receives IDs #1\n\r");
          Puts3("CAN-Buffer 2 receives IDs with MSB (ID28) set (e.g. 400)\n\r");
          Puts3("CAN-Buffer 5 receives all other IDs\n\r");
          Puts3("CAN-Buffer 6 sends data\n\r");
          Puts3("\n\r\n\r");
          break;
          
        case 'I': /* enable/disable CAN-Receive-IRQs */
          if (RIER1)
          {
            RIER1 = 0;
            Puts3("CAN Receive IRQs disabled\n\n");
          }
          else
          {
            RIER1 = 0xffff;
            Puts3("CAN Receive IRQs enabled\n\n");
          }
          break;
          
        default:  /* any other character => to send via CAN buffer #6  */
          Puts3("send via CAN (buffer #6)    ==>     ID: 0x"); 
          binasci16(STD2MSG(IDR1(6)), HEX, '0');
          Puts3("   Data: ");
          Putch3(ch);
          DLCR1(buffer6) = 1;	             /* 1 Byte to send via CAN           */
          DTR1_WORD(buffer6,0) = ch;         /* Character to send                */
          TREQR1 = 1 << buffer6;             /* transmition request: TREQ6 = 1   */
          timeout = 0;
          while (   TREQR1_TREQ6             /* <<< wait until trx is completed  */
                 && ! CSR1_HALT              /* or busoff                        */
                 && (++timeout < TIMEOUT) ); /* or timeout                       */

          if (timeout == TIMEOUT)            /* if timeout...                    */
          {
            TCANR1 = 1 << buffer6;           /* ... then cancel Transmit         */
            Puts3("  timeout !");
          }
          else
          if (CSR1_HALT)                     /* if busoff... */
          {
            Puts3("  BusOff !");
          }        

          Puts3("\n");
            
          break;
 
      } // switch (ch)
    } // if (SSR1_RDRF)

    if (CSR1_HALT)                  /* === simple Bus-off Handler ===          */
    {                               /* if busoff...                            */
      CAN_init();                   /* ... initialize CAN                      */ 
      CAN_buffer1();                /* ... initialize used message-buffers     */
      CAN_buffer2();                /* ... initialize used message-buffers     */
      CAN_buffer5();                /* ... initialize used message-buffers     */
      CAN_buffer6();                /* ... initialize used message-buffers     */
    }

  } // while(1)
} // main


__interrupt void IRQ_CANRX (void)
{
  buffer = 0; buffercnt = 1;

  while(RCR1) /* get all received messages */
  {
    if (RCR1 & buffercnt)       /* check if buffer has received message */
    {
      do
      {
        ROVRR1 = ~buffercnt;      /* clear Overrun-flag */

        GetCanData (buffer, CAN_Receive_Data); /* Readout Message Buffer */
        CAN_Receive_ID = IDR1(buffer);
      } 
      while (ROVRR1 & buffercnt);   /* check for overrun: 
                                       Theoretically it may happen that another CAN-message 
                                       is received while data was stored */

      RCR1 = ~buffercnt;            /* clear Reception-flag */

      Puts3("CAN message received by buffer #");
      binasci4(buffer, DEC, ' ');

      Puts3("  ID: 0x");
      CAN_Receive_ID =   STD2MSG(CAN_Receive_ID); /* reordering of ID */
    
      binasci16(CAN_Receive_ID, HEX, '0');

      Puts3("   Data: ");      
      for (DataPos=1; DataPos<=CAN_Receive_Data[0]; DataPos++)
      {
        binasci8(CAN_Receive_Data[DataPos], HEX, '0');	// send to uart
        Putch3(32);
      }
      Puts3("\n");
    } //if (RCR1 & buffercnt)
    buffer = buffer + 1; 
    buffercnt = buffercnt << 1;
    if (buffer == 16)
    {
      buffer = 0;
      buffercnt = 1;
    }
  }// while
//  Puts3("\n\r");
}//IRQ  

